@@ -1,4 +1,4 @@
-#    $Id: Nominet.pm,v 1.7 2014/08/16 17:02:04 pete Exp $
+#    $Id: Nominet.pm,v 1.14 2015/11/06 14:39:18 pete Exp $
 #
 #    This program is free software; you can redistribute it and/or modify
 #    it under the terms of the GNU General Public License as published by
@@ -29,9 +29,7 @@ use constant EPP_XMLNS	=> 'urn:ietf:params:xml:ns:epp-1.0';
 use vars qw($Error $Code $Message);
 
 BEGIN {
-	our ($VERSION, @ISA);
-	$VERSION    = '0.02';
-	@ISA        = qw(Net::EPP::Simple Exporter);
+	our $VERSION = '0.03';
 }
 
 # file-scoped lexicals
@@ -46,19 +44,19 @@ my $Debug     = 0;
 
 =head1 Name
 
-Net::EPP::Registry::Nominet - a simple client interface to Nominet EPP
-jobs
+Net::EPP::Registry::Nominet - a simple client interface to the Nominet EPP
+service
 
 =head1 Synopsis
 
-	#!/usr/bin/perl
 	use strict;
+	use warnings;
 	use Net::EPP::Registry::Nominet;
 
 	my $epp = Net::EPP::Registry::Nominet->new (
 		user  =>  'MYTAG',
 		pass  =>  'mypass'
-	) or die ('Could not login to EPP server: ', $epp->get_error);
+	) or die ('Could not login to EPP server: ', $Net::EPP::Registry::Nominet::Message);
 
 	my $dom = 'foo.co.uk';
 	
@@ -82,8 +80,8 @@ jobs
 
 L<Nominet|http://www.nominet.org.uk/> is the organisation in charge of
 domain names under the .uk TLD.  Historically it used cryptographically
-signed email communications with registrars to provision domains (and
-still does). More recently (since 2010) it has instituted an EPP system
+signed email communications with registrars to provision domains.
+More recently (since 2010) it has instituted an EPP system
 which is sufficiently different from standard EPP that none of the
 standard modules will work seamlessly with it.
 
@@ -104,10 +102,10 @@ END {}
 	my $epp = Net::EPP::Registry::Nominet->new (
 		user  =>  'MYTAG',
 		pass  =>  'mypass'
-	) or die ('Could not login to EPP server: ', $epp->get_error);
+	) or die ('Could not login to EPP server: ', $Net::EPP::Registry::Nominet::Message);
 
 The constructor for C<Net::EPP::Registry::Nominet> has the same
-general form as the one for C<Net::EPP::Simple>, but with the following
+general form as the one for L<Net::EPP::Simple>, but with the following
 exceptions:
 
 =over
@@ -176,12 +174,13 @@ sub new {
 
 	my $self = Net::EPP::Client->new(%params);
 	unless ($self->{timeout}) { $self->{timeout} = $params{timeout}; }
+	$self->{debug} = $Debug > 1 ? 1 : 0; # for parent
 
 	# Set the default years.
 	$self->{def_years} = 2;
 	if (defined $params{def_years}) {
 		my $years = scalar $params{def_years};
-		if ($years and $years =~ /^[0-9]+$/) {
+		if ($years =~ /^[0-9]+$/) {
 			if ($years > 0 and $years < 11) {
 				$self->{def_years} = $years;
 			} else {
@@ -261,9 +260,11 @@ sub login {
 		$login->svcs->appendChild($foo);
 	} else {
 		# Standard schemas and extensions
-		for my $ns ('epp', 'eppcom', 'domain', 'host', 'contact') {
+		for my $ns ('epp', 'eppcom', 'domain', 'host', 'contact', 'secDNS') {
 			my $el = $login->createElement('objURI');
-			$el->appendText("urn:ietf:params:xml:ns:$ns-$EPPVer");
+			my $ver = $EPPVer;
+			$ver = 1.1 if $ns eq 'secDNS';
+			$el->appendText("urn:ietf:params:xml:ns:$ns-$ver");
 			$login->svcs->appendChild($el);
 		}
 		# Extensions go here
@@ -293,7 +294,7 @@ sub login {
 
 =head1 Availability checks
 
-The availability checks work similarly to C<Net::EPP::Simple> except
+The availability checks work similarly to L<Net::EPP::Simple> except
 that they return an array with three elements. The first element is the
 availability indicator as before (0 if provisioned, 1 if available,
 undef on error) and the second element is the abuse counter which shows
@@ -530,7 +531,15 @@ registration.
 		nameservers  => {
 			'nsname0'  => "ns1.bar.co.uk",
 			'nsname1'  => "ns2.bar.co.uk"
-		}
+		},
+		sedDNS       => [
+			{
+				keyTag     => 123,
+				alg        => 5,
+				digestType => 1,
+				digest     => '8A9CEBB665B78E0142F1CEF47CC9F4205F600685'
+			}
+		]
 	};
 	my ($res) = $epp->create_domain ($domain);
 	if ($res) {
@@ -619,6 +628,21 @@ sub create_domain {
 	$name->appendChild ($pw);
 	$obj->appendChild ($name);
 
+
+	# DNSSEC
+	my $exttype = 'secDNS';
+	if ($domain->{$exttype}) {
+		my @spec  = $self->spec ($exttype);
+		my $obj2  = $frame->addObject (@spec);
+		for my $dsrec (@{$domain->{$exttype}}) {
+			$self->_add_dsrec ($obj2, $frame, $dsrec);
+		}
+
+		my $extension = $frame->command->new ('extension');
+		$extension->appendChild ($obj2);
+		$frame->command->insertAfter ($extension, $frame->getCommandNode);
+	}
+
 	# Request complete, so send the frame
 	$frame->getCommandNode->appendChild ($obj);
 
@@ -741,6 +765,18 @@ sub _add_nsaddr {
 	return;
 }
 
+sub _add_dsrec {
+	my ($self, $name, $frame, $dsrec) = @_;
+	my $ds = $frame->createElement ('secDNS:dsData');
+	for my $key (qw/ keyTag alg digestType digest /) {
+		my $field = $frame->createElement ("secDNS:$key");
+		$field->appendText ($dsrec->{$key});
+		$ds->appendChild ($field);
+	}
+	$name->appendChild ($ds);
+	return;
+}
+
 =head1 Modify objects
 
 The domains, contacts and hosts once created can be modified using
@@ -756,8 +792,6 @@ changes like this:
 		'add'          => { ns => ['ns1.newhost.com', 'ns2.newhost.com'] },
 		'rem'          => { ns => ['ns1.oldhost.net', 'ns2.oldhost.net'] },
 		'chg'          => {},
-		'first-bill'   => 'th',
-		'recur-bill'   => 'th',
 		'auto-bill'    => 21,
 		'auto-period'  => 5,
 		'next-bill'    => '',
@@ -770,11 +804,26 @@ You cannot use C<chg> to change nameservers or extension fields. The C<chg>
 entry is only used to move a domain between registrants with the same
 name.
 
-The extension fields can only be set outside of the add, rem and chg
-fields. The supported extensions in this module are:
-first-bill, recur-bill, auto-bill, auto-period, next-bill, next-period
-and notes. All of these are scalars aside from notes which is an array
-ref.
+The C<add> and C<rem> groups are also used to add and remove DS records.
+eg:
+
+	my $changes = {
+		'name'         => 'foo.co.uk',
+		'add'          => {
+			secDNS => [{ 
+				keyTag     => 25103,
+				alg        => 5,
+				digestType => 1,
+				digest     => '8A9CEBB665B78E0142F1CEF47CC9F4205F600685'
+			}]
+		}
+		'rem'          => {}
+	};
+
+The extension fields can only be set outside of the add, rem and
+chg fields. The supported extensions in this module are: C<auto-bill>,
+C<auto-period>, C<next-bill>, C<next-period>, C<renew-not-required> and
+C<notes>. All of these are scalars aside from C<notes> which is an array ref.
 
 C<update_domain()> returns undef on failure, 1 on success.
 
@@ -822,8 +871,8 @@ sub modify_domain {
 	# Extensions
 	@spec = $self->spec ('domain-nom-ext');
 	$obj  = $frame->addObject (@spec);
-	for my $field (qw/ first-bill recur-bill auto-bill auto-period
-			next-bill next-period reseller /) {
+	for my $field (qw/ auto-bill auto-period
+			next-bill next-period renew-not-required reseller /) {
 		next unless ($data->{$field});
 		my $name = $frame->createElement("domain-nom-ext:$field");
 		$name->appendText ($data->{$field});
@@ -836,8 +885,26 @@ sub modify_domain {
 			$obj->appendChild ($name);
 		}
 	}
+	# DNSSEC
+	# Nominet does not support MaxSigLife which is the only possible use
+	# for 'chg', so do not cater for it here (yet).
+	# 
+	my $exttype = 'secDNS';
+	@spec  = $self->spec ($exttype);
+	my $obj2  = $frame->addObject (@spec);
+	for my $action ('rem', 'add') {
+		if ($data->{$action}->{$exttype}) {
+			$name = $frame->createElement ("$exttype:$action");
+			for my $dsrec (@{$data->{$action}->{$exttype}}) {
+				$self->_add_dsrec ($name, $frame, $dsrec);
+			}
+			$obj2->appendChild ($name);
+		}
+	}
+
 	my $extension = $frame->command->new ('extension');
 	$extension->appendChild ($obj);
+	$extension->appendChild ($obj2);
 	$frame->command->insertAfter ($extension, $frame->getCommandNode);
 
 	my $response = $self->_send_frame ($frame);
@@ -874,7 +941,7 @@ changes like this:
 	};
 	my $res = $epp->update_contact ($changes) or die $epp->get_reason;
 
-Note that this differs from the syntax of C<Net::EPP::Simple> where that
+Note that this differs from the syntax of L<Net::EPP::Simple> where that
 takes the stock C<add>, C<rem> and C<chg> elements.
 
 It returns undef on failure, 1 on success.
@@ -908,7 +975,7 @@ sub modify_contact {
 	#   my $frame = $self->_generate_update_contact_frame($data);
 	# but it won't work because of present, but empty, add/rem/chg
 	# elements. Equally we cannot do this:
-    #   my $frame = Net::EPP::Frame::Command::Update::Contact->new;
+	#   my $frame = Net::EPP::Frame::Command::Update::Contact->new;
 	#   $frame->setContact ( $cont );
 	# so instead it needs this extra chunk of code which follows:
 
@@ -1088,15 +1155,11 @@ sub _info {
 
 	if ($type eq 'domain') {
 		my $extra = $response->getNode('domain-nom-ext:infData');
+		my $extra2 = $response->getNode('secDNS:infData');
 		return $self->_domain_infData_to_hash($infData, $extra);
 	} elsif ($type eq 'contact') {
+		$self->_clean_addr($infData);
 		my $this = $self->_contact_infData_to_hash($infData);
-		# Strip out the strange empty addr entries created by
-		# Net::EPP::Simple
-		for my $subtype ('int', 'loc') {
-			delete $this->{postalInfo}->{$subtype}->{addr}->{''} if
-				$this->{postalInfo}->{$subtype};
-		}
 		# Add in the Nominet extras (reg, rather than contact)
 		my $extra = $response->getNode('contact-nom-ext:infData');
 		return $self->_merge_contact_infData ($this, $extra);
@@ -1111,9 +1174,9 @@ sub _domain_infData_to_hash {
 	my $hash = $self->_node_to_hash ($infData, ['registrant',
 		'clID', 'crID', 'crDate', 'exDate', 'name', 'roid']);
 
-	my $extrahash = $self->_node_to_hash ($extra, ['first-bill',
-	'recur-bill', 'auto-bill', 'next-bill', 'auto-period',
-	'next-period', 'reg-status', 'notes', 'reseller']);
+	my $extrahash = $self->_node_to_hash ($extra, [
+	'auto-bill', 'next-bill', 'auto-period',
+	'next-period', 'reg-status', 'renew-not-required', 'notes', 'reseller']);
 
 	for (keys %$extrahash) {
 		$hash->{$_} = $extrahash->{$_};
@@ -1127,6 +1190,18 @@ sub _domain_infData_to_hash {
 	return $hash;
 }
 
+sub _clean_addr {
+	my ($self, $infData) = @_;
+	# Remove the whitespace text nodes from the addresses as
+	# Net::EPP::Simple does not handle them nicely
+	for my $addr ($infData->getElementsByTagName('contact:addr')) {
+		for my $child ($addr->childNodes) {
+			$addr->removeChild ($child)
+				unless $child->nodeType == XML::LibXML::XML_ELEMENT_NODE;
+		}
+	}
+}
+
 sub _merge_contact_infData {
 	my ($self, $old, $extra) = @_;
 
@@ -1218,7 +1293,7 @@ The hello method takes no arguments. It returns 1 on success, undef
 otherwise.
 
 This performs much the same function as the ping method of
-Net:EPP::Simple (which could be used instead) but provides more
+L<Net::EPP::Simple> (which could be used instead) but provides more
 extensive error handling.
 
 =cut
@@ -1229,7 +1304,7 @@ sub hello {
 		warn "Hello attempt while disconnected\n" if $Debug;
 		return undef;
 	}
-	my $frame = Net::EPP::Frame::Hello->new->toString;
+	my $frame = Net::EPP::Frame::Hello->new;
 
 	warn "Sending XML = \n" . $frame . "\n" if $Debug;
 	my $greeting = $self->request($frame);
@@ -1278,6 +1353,12 @@ sub spec {
 			'http://www.nominet.org.uk/epp/xml/domain-nom-ext-1.2',
 			'http://www.nominet.org.uk/epp/xml/domain-nom-ext-1.2 domain-nom-ext-1.2.xsd');
 	}
+	if ($type eq 'secDNS') {
+		my $ver = 1.1;
+		return ($type,
+			"urn:ietf:params:xml:ns:secDNS-$ver",
+			"urn:ietf:params:xml:ns:secDNS-$ver secDNS-$ver.xsd");
+	}
 	if ($type eq 'contact') {
 		return ($type,
 			"urn:ietf:params:xml:ns:contact-$EPPVer",
@@ -1445,11 +1526,7 @@ sub _send_frame {
 =over
 
 =item * The poll, fork, handshake, lock, tag list and reseller operations
-are not yet supported. Nor is there any DNSSEC facility yet.
-
-=item * Retrieving contact info can generate warnings from the
-superclass. Ideally this processing should be done locally (within this
-class) to avoid these warnings.
+are not yet supported.
 
 =item * Much more extensive tests should be performed.
 
@@ -1477,7 +1554,7 @@ Pete Houston <cpan@openstrike.co.uk>
 
 =head1 Licence
 
-This software is copyright (c) 2013 by Pete Houston. It is released
+This software is copyright (c) 2013-2015 by Pete Houston. It is released
 under the Artistic Licence (version 2) and the
 GNU General Public Licence (version 2).
 
